%==========================================================================
% [plt_lin_nomscen.m] Generates Figures 5.4 & 5.5
%   Generate nominal linear step response diagrams
%
%   Author: T.S.C. Pollack
%   Copyright (2024) T.S.C. Pollack
%==========================================================================

clear all; close all; clc;

%% User settings

% Add function paths
addpath('_MISC'); addpath('../0_MISC')

% Go to data directory
cd('..\..\..\_FIGDATA\2_LIN\1_NOMINAL\h10kft_V500fts');

% Set plotting theme
clr_lst   = color_theme();
clr_order = [3,1,5];
style_lst = {'-','-','-','-'};

% Set plotting horizon
Tplt = [0.8,9];

% Scan data
dir_lst = dir; dir_lst = dir_lst(3:end);

% Calculate optimal control response in steady-state
load(strcat(dir_lst(1).name,'\umbrella.mat'));
qss = y_sim.Data(end,2);
[L_opt,dh_opt,dlef_opt,alpha_opt,sigma_opt] = ...
                            ss_opt_calculator(A,B,C,D,qss);
cd_opt = sigma_opt(1)*CDref;

%% Plot
FIG = figure(1); FIG.Position = [300 59 560*2 380*1.5];

j = 0;

for i = 1:length(dir_lst)
    
    j = j + 1;

    load(strcat(dir_lst(j).name,'\umbrella.mat'));

    T = ref.Time;

    if j == 1
        sp1 = subplot(3,2,1);
        sp1.Position = sp1.Position+ [-0.02 -0.01 0.05 0.04];
        plot(sp1,T,ref.Data(:,2),'r--'); grid on; hold on;
    end
        
    plot(sp1,T,y_sim.Data(:,2),'color',clr_lst(clr_order(j),:),...
        'LineStyle',style_lst{j});
    ylabel(sp1,'Pitch rate (deg/s)','Interpreter','latex','FontSize',12);
    set(sp1,'TickLabelInterpreter','latex')
    set(sp1,'XTickLabel',[])
    xlim(Tplt);

    % Add legend
    legend(sp1,'Reference','Trim LEF','Static allocation','Optimal control', ...
        'Orientation','horizontal','NumColumns',2, ...
        'Interpreter','latex','Location','NorthEast','FontSize',12);

    cd = y_sim.Data(:,5);

    if j == 1
        sp2 = subplot(3,2,5);
        sp2.Position = sp2.Position+ [-0.02 -0.01 0.05 0.04];
    end
    
    plot(sp2,T,cd,'color',clr_lst(clr_order(j),:),'LineStyle', ...
        style_lst{j}); grid on; hold on;
    plot(sp2,T,cd_opt*ones(size(T)),'k--');
    xlabel(sp2,'Time (s)','Interpreter','latex','FontSize',12);
    ylabel(sp2,'$C_D$ (-)','Interpreter','latex','FontSize',12);
    set(sp2,'TickLabelInterpreter','latex')
    xlim(Tplt);

    if j == 1
        axzoom_cd = axes('position',[.275 .125 .2 .15]); 
        box on; hold on; grid on;
        zoomidx = (T < T(end)) & (T > T(end)-2);
    end
    
    plot(axzoom_cd,T(zoomidx),cd(zoomidx),'color',...
        clr_lst(clr_order(j),:),'LineStyle',style_lst{j});
    axis tight;
    plot(axzoom_cd,T(zoomidx),cd_opt*ones(size(T(zoomidx))),'k--');
    ylim(axzoom_cd,[0.0205,0.02205])
    set(axzoom_cd,'TickLabelInterpreter','latex')
    set(axzoom_cd,'XTickLabel',[])

    if j == 1
        sp3 = subplot(3,2,2);
        sp3.Position = sp3.Position+ [-0.02 -0.01 0.05 0.04];
    end
    
    plot(sp3,T,y_sim.Data(:,1),'color',clr_lst(clr_order(j),:),...
        'LineStyle',style_lst{j}); grid on; hold on;
    plot(sp3,T,alpha_opt*ones(size(T)),'k--');
    ylabel(sp3,'Angle-of-attack (deg)','Interpreter','latex','FontSize',12);
    set(sp3,'TickLabelInterpreter','latex')
    set(sp3,'XTickLabel',[])
    xlim(Tplt);

    if j == 1
        sp4 = subplot(3,2,4);
        sp4.Position = sp4.Position+ [-0.02 -0.01 0.05 0.04];
    end
    
    plot(sp4,T,surfaces(:,2),'color',clr_lst(clr_order(j),:),...
        'LineStyle',style_lst{j}); grid on; hold on;
    plot(sp4,T,dlef_opt*ones(size(T)),'k--');
    ylabel(sp4,'$\delta_{LEF}$ (deg)','Interpreter','latex','FontSize',12);
    set(sp4,'TickLabelInterpreter','latex')
    set(sp4,'XTickLabel',[])
    xlim(Tplt);

    if j == 1
        sp6 = subplot(3,2,3);
        sp6.Position = sp6.Position+ [-0.02 -0.01 0.05 0.04];
    end
    
    plot(sp6,T,surfaces(:,1),'color',clr_lst(clr_order(j),:),...
        'LineStyle',style_lst{j}); grid on; hold on;
    plot(sp6,T,dh_opt*ones(size(T)),'k--');
    ylabel(sp6,'$\delta_h$ (deg)','Interpreter','latex','FontSize',12);
    set(sp6,'TickLabelInterpreter','latex')
    set(sp6,'XTickLabel',[])
    xlim(Tplt);

    if j == 1
        sp8 = subplot(3,2,6);
        sp8.Position = sp8.Position+ [-0.02 -0.01 0.05 0.04];
    end
    
    plot(sp8,T,L.Data,'color',clr_lst(clr_order(j),:),'LineStyle',...
        style_lst{j}); grid on; hold on;
    plot(sp8,T,L_opt*ones(size(T)),'k--');
    xlabel(sp8,'Time (s)','Interpreter','latex','FontSize',12);
    ylabel(sp8,'$L$ (-)','Interpreter','latex','FontSize',12);
    set(sp8,'TickLabelInterpreter','latex')
    xlim(Tplt);

    if j == 1
        axzoom_L = axes('position',[.715 .125 .2 .15]);
        box on; hold on; grid on;
    end
    
    plot(axzoom_L,T(zoomidx),L.Data(zoomidx),'color',...
        clr_lst(clr_order(j),:),'LineStyle',style_lst{j});
    axis tight;
    plot(axzoom_L,T(zoomidx),L_opt*ones(size(T(zoomidx))),'k--');
    ylim(axzoom_L,[2.25,2.45])
    set(axzoom_L,'TickLabelInterpreter','latex')
    set(axzoom_L,'XTickLabel',[])
    
end
