%==========================================================================
% [main_lin.m] Main file to generate linear incremental control allocation 
% and optimal control results
%
%   Author: T.S.C. Pollack
%   Copyright (2024) T.S.C. Pollack
%==========================================================================

clear all; close all; clc; 

%% User settings

% Load bare airframe short-period model
load('../_MODELS/LINSS_F16_UOM_h10kft_V450fts_XCG0.382MAC.mat');

% Specify export file name
fname = 'LIN_F16_INOC_TSS_h10kft_V450fts_kp5.mat';

% Script settings
run_sim     = 1;
run_batch   = 0;
plt_results = 1;
save_file   = 0;

% Controller mode
ctrlmode     = 'inca_tss';

% Actuator saturation on/off
actsat_mode  = 1;

% Pilot input gain
kcmd = 1;

% Performance objective gains
kp = 5;
kr = 0.5;

% Parametric variations
dtab = [-0.5,0,0.5];

% Simulation settings
delta_T   = 1e-2; 
T_end     = 10;

%% Preliminaries

% Add paths
addpath(genpath('_FUNCTIONS/')); addpath(genpath('_SIMULINK/'));

% Rename
ba_lon_sys = ba_sp_sys;

% Get dimensions
nu  = [2 3]; 
nuu = size(nu,2); 
ny  = size(ba_lon_sys,1);

% Set broken-loop flags
bl_flag_dh   = 0;
bl_flag_dlef = 0;
bl_flag_nu   = 0;
bl_flag_dd   = 0;

% Decimal no
ndec = 3;

% Assign parametric variations
dAtab{1,1} = dtab;
dAtab{1,2} = dtab;
dAtab{2,1} = dtab;
dAtab{2,2} = dtab;
dBtab{1,1} = dtab;
dBtab{1,2} = dtab;
dBtab{2,1} = dtab;
dBtab{2,2} = dtab;
dCtab{1,1} = dtab;
dCtab{1,2} = dtab;
dDtab{1,1} = dtab;
dDtab{1,2} = dtab;

%% Set controller parameters

% LEF active yes/no
lef_act_flag = ~strcmp(ctrlmode,'nolef');

% Specify actuator saturation limits
if actsat_mode
    ulim(1,1) = -25-u0(1); ulim(1,2) = 25-u0(1);
    ulim(2,1) = 0-u0(4);   ulim(2,2) = 25-u0(4);
elseif ~actsat_mode
    ulim = [-inf,inf;-inf,inf];
end

% Desired dynamics
num_tf_ddq = [16 16 0];
den_tf_ddq = [0.2936 2.349 4.698];

Gdes = minreal(ss([tf(num_tf_ddq,den_tf_ddq);...
                    tf(num_tf_ddq(1:2),den_tf_ddq)]));

% Regulator gains
K_p = 5; K_i = 0;

% Performance objective normalization
CDref = 1e-2;
Wp    = blkdiag(1/(25*2),1/25)^2;
Wr    = blkdiag(0.1,1);

% Define synthesis airframe system
A      = ba_lon_sys.A;
B      = ba_lon_sys.B;
C      = [1/CDref*ba_lon_sys.C(5,:)',zeros(2,2)]';
D      = [1/CDref*ba_lon_sys.D(5,:)',kp*sqrt(Wp)]';
ba_sys = ss(A,B,C,D);

% Generate optimal restoring control law
By = B(2,:);

% Primary state feedback matrix - assuming perfect cancellation of Ax
TT = [K_p*[0,1] + A(2,:), By];

% Generate restoring control law
if strcmp(ctrlmode,'inoc')    
    [P,N,Kx,Kr,X,H] = indare(A,B,C,D,kr^2*Wr,TT,By,delta_T);
elseif strcmp(ctrlmode,'inoc_tss')    
    [P,N,Ks,Kx,Kr,X,H] = indare_tss(A,B,C,D,kr^2*Wr,TT,By,delta_T);
elseif strcmp(ctrlmode,'inca')    
    [P,N,Kx,Kr] = kica(C,D,kr^2*Wr,By);
elseif strcmp(ctrlmode,'inca_tss')    
    [P,N,Ks,Kx,Kr] = kica_tss(C,D,kr^2*Wr,By);
elseif strcmp(ctrlmode,'nolef')    
    [P,N,Kx,Kr] = kica(C,D,kr^2*Wr,[By(1),0]);
end

% Define plant performance matrices
C_s = C; D_s = D;

%% Simulate
fsname = strcat('_RESULTS/',fname);

if run_batch    
    if contains(ctrlmode,'tss')
        batch_lintss;
    else
        batch_lin;
    end
elseif run_sim
    if contains(ctrlmode,'tss')
        sim('F16_LTI_INOC_TSS.slx');
    else
        sim('F16_LTI_INOC.slx');
    end
end

% Save file
if save_file
    save(fsname);
end

%% Plot results

if plt_results

    close all;

    clr_lst = [[0, 0.4470, 0.7410];[0.8500, 0.3250, 0.0980];[0.4660, 0.6740, 0.1880]; ...
        [0.4940, 0.1840, 0.5560];  [0.6350, 0.0780, 0.1840];[0.25, 0.25, 0.25]; ...
        [0.75, 0.75, 0];[0, 0.75, 0.75];[0.9290, 0.6940, 0.1250]];

    T = ref.Time;

    FIG_step_nom = figure(1); FIG_step_nom.Position = [300 59 560*2 380*2];

    sp1 = subplot(3,2,1);
    plot(T,ref.Data(:,1),'r--'); grid on; hold on;
    plot(T,ref.Data(:,2),'color',clr_lst(3,:));
    ylabel('Pitch rate (deg/s)','Interpreter','latex','FontSize',12);
    set(gca,'TickLabelInterpreter','latex')
    set(gca,'XTickLabel',[])

    sp1.Position = sp1.Position+ [-0.02 -0.02 0.05 0.05];

    cd = y_sim.Data(:,5);

    sp2 = subplot(3,2,5);
    plot(T,cd,'color',clr_lst(3,:)); grid on; hold on;
    xlabel('Time (s)','Interpreter','latex','FontSize',12);
    ylabel('$C_D$ (-)','Interpreter','latex','FontSize',12);
    set(gca,'TickLabelInterpreter','latex')
    sp2.Position = sp2.Position+ [-0.02 -0.02 0.05 0.05];

    sp3 = subplot(3,2,2);
    plot(T,y_sim.Data(:,1),'color',clr_lst(3,:)); grid on; hold on;
    ylabel('Angle-of-attack (deg)','Interpreter','latex','FontSize',12);
    set(gca,'TickLabelInterpreter','latex')
    set(gca,'XTickLabel',[])
    sp3.Position = sp3.Position + [-0.02 -0.02 0.05 0.05];

    sp4 = subplot(3,2,4);
    plot(T,surfaces(:,2),'color',clr_lst(3,:)); grid on; hold on;
    ylabel('$\delta_{LEF}$ (deg)','Interpreter','latex','FontSize',12);
    set(gca,'TickLabelInterpreter','latex')
    set(gca,'XTickLabel',[])
    sp4.Position = sp4.Position + [-0.02 -0.02 0.05 0.05];

    sp5 = subplot(3,2,6);
    plot(T,L.Data,'color',clr_lst(3,:)); grid on; hold on;
    xlabel('Time (s)','Interpreter','latex','FontSize',12);
    ylabel('$L$ (-)','Interpreter','latex','FontSize',12);
    set(gca,'TickLabelInterpreter','latex')
    sp5.Position = sp5.Position + [-0.02 -0.02 0.05 0.05];

    sp6 = subplot(3,2,3);
    plot(T,surfaces(:,1),'color',clr_lst(3,:)); grid on; hold on;
    ylabel('$\delta_h$ (deg)','Interpreter','latex','FontSize',12);
    set(gca,'TickLabelInterpreter','latex')
    set(gca,'XTickLabel',[])
    sp6.Position = sp6.Position + [-0.02 -0.02 0.05 0.05];
    
end

